"""Asset exports"""
ROOT_SPINNER_URL = (
    "https://github.com/MarcSkovMadsen/awesome-panel/raw/master/assets/images/spinners/"
)
SPINNER_AQUA = ROOT_SPINNER_URL + "spinner_aqua.png"
SPINNER_PANEL_BLINK_LIGHT_400_340 = ROOT_SPINNER_URL + "spinner_panel_blink_light_400_340.gif"
SPINNER_PANEL_BREATH_LIGHT_400_170 = ROOT_SPINNER_URL + "spinner_panel_breath_light_400_170.gif"
SPINNER_PANEL_BREATH_LIGHT_400_340 = ROOT_SPINNER_URL + "spinner_panel_breath_light_400_340.gif"
SPINNER_PANEL_GREEN_DARK = ROOT_SPINNER_URL + "spinner_panel_green_dark.gif"
SPINNER_PANEL_GREEN_DARK_STATIC = ROOT_SPINNER_URL + "spinner_panel_green_dark_static.gif"
SPINNER_PANEL_GREEN_LIGHT = ROOT_SPINNER_URL + "spinner_panel_green_light.gif"
SPINNER_PANEL_GREEN_LIGHT_STATIC = ROOT_SPINNER_URL + "spinner_panel_green_light_static.gif"
SPINNER_PANEL_ROTATE_400_400 = ROOT_SPINNER_URL + "spinner_panel_rotate_400_400.gif"
SPINNER_PANEL_STATIC_LIGHT = ROOT_SPINNER_URL + "spinner_panel_static_light.png"
SPINNER_PANEL_STATIC_LIGHT_400_340 = ROOT_SPINNER_URL + "spinner_panel_static_light_400_340.gif"
